<?php
    # GeoREP - Gestionnaire des couches de la GéoBASE
    # Copyright (C) 2004 - Michel Wurtz - MAAPAR/SDSI/CERI/DIG
    # basé sur Mantis (http://www.mantisbt.org)
    # modifié par Sylvain Perrinel - 2007 - MAAPAR/SDSI/CERI/DIG
    # Copyright (C) 2000 - 2002 Kenzaburo Ito - kenito@300baud.org
    # Copyright (C) 2002 - 2004 Mantis Team - mantisbt-dev@lists.sourceforge.net
    # This program is distributed under the terms and conditions of the GPL
    # See the README and LICENSE files for details

    # --------------------------------------------------------
    # $Id: database_api.php,v 1.20 2004/01/31 15:10:33 vboctor Exp $
    # --------------------------------------------------------

    ###########################################################################
    # Database
    ###########################################################################

    # This is the general interface for all database calls.
    # Use this as a starting point to port to other databases

    # An array in which all executed queries are stored.  This is used for profiling
    $g_queries_array = array();
    
    # Stores whether a database connection was succesfully opened.
    $g_db_connected = false;

    # --------------------
    # Make a connection to the database
    function db_connect( $p_hostname, $p_username, $p_password, $p_port, $p_database_name ) {
        global $g_db_connected;
        //echo "host=".$p_hostname." port=".$p_port." user=". $p_username." password=".$p_password." dbname=".$p_database_name;
        $t_result = @pg_connect( "host=".$p_hostname." port=".$p_port." user=". $p_username." password=".$p_password." dbname=".$p_database_name );
        if ( !$t_result ) {
            return false;
        }
        
        $g_db_connected = true;

        return true;
    }

    # --------------------
    # Make a persistent connection to the database
    function db_pconnect( $p_hostname, $p_username, $p_password, $p_port, $p_database_name ) {
        global $g_db_connected;
       
        $t_result = @pg_pconnect( "host=".$p_hostname." port=".$p_port." user=". $p_username." password=".$p_password." dbname=".$p_database_name );

        if ( !$t_result ) {
            return false;
        }
        
        $g_db_connected = true;

        return true;
    }
    
    # --------------------
    # Returns whether a connection to the database exists
    function db_is_connected() {
        global $g_db_connected;
        
        return $g_db_connected;
    }

    # --------------------
    # execute query, requires connection to be opened
    # If $p_error_on_failure is true (default) an error will be triggered
    #  if there is a problem executing the query.
    # If $p_error_on_failure is false, false will be returned if there is a
    #  problem.  This should be used very infrequently.  It was added to allow
    #  the admin script to check whether a table exists.
    function db_query( $p_query, $p_error_on_failure=true ) {
        global $g_queries_array;

        array_push ( $g_queries_array, $p_query );
    //echo $p_query;
        $t_result = pg_query( $p_query );

        # @@@ remove p_error_on_failure and use @ in every caller that used to use it
        if ( !$t_result && $p_error_on_failure ) {

            return false;
        } else {
            return $t_result;
        }
    }

    # --------------------
    function db_select_db( $p_db_name ) {
        global $g_db_connected;
        $t_result = @pg_select_db( $p_db_name );

        if ( !$t_result ) {
            db_close();
            # we got the wrong database so we're basically not connected.
            # this prevents the html functions from trying to display db stuff
            $g_db_connected = false;

            return false;
        }

        return $t_result;
    }

    # --------------------
    function db_num_rows( $p_result ) {
        return pg_num_rows( $p_result );
    }

    # --------------------
    function db_affected_rows($p_result) {
        return pg_affected_rows($p_result);
    }

    # --------------------
    function db_fetch_array( $p_result ) {
        return pg_fetch_array( $p_result );
    }

    # --------------------
    function db_result( $p_result, $p_index1=0, $p_index2=0 ) {
        if ( $p_result && ( db_num_rows( $p_result ) > 0 ) ) {
            return pg_result( $p_result, $p_index1, $p_index2 );
        } else {
            return false;
        }
    }

    # --------------------
    # return the last inserted id
    function db_insert_id($p_schema, $p_name_seq) {
        $t_return = db_result(db_query('SELECT CURRVAL(\''.$p_schema.'."'.$p_name_seq.'_seq"\'::regclass) AS id'));
        
        //Et on retourne le champ id qui contiendra la valeur du dernier id insérée
        return (int)$t_return['id'];
    }

    # --------------------
    function db_field_exists( $p_field_name, $p_table_name, $p_db_name = '') {
        global $g_database_name;

        if ($p_db_name == '') {
            $p_db_name = $g_database_name;
        }

        $fields = pg_list_fields($p_db_name, $p_table_name);
        $columns = pg_num_fields($fields);
        for ($i = 0; $i < $columns; $i++) {
          if ( pg_field_name( $fields, $i ) == $p_field_name ) {
              return true;
          }
        }

        return false;
    }

    # --------------------
    # Check if there is an index defined on the specified table/field and with
    # the specified type.
    #
    # $p_table: Name of table to check
    # $p_field: Name of field to check
    # $p_key: key type to check for (eg: PRI, MUL, ...etc)
    function db_key_exists_on_field( $p_table, $p_field, $p_key ) {
        $c_table = db_prepare_string( $p_table );
        $c_field = db_prepare_string( $p_field );
        $c_key   = db_prepare_string( $p_key );

        $query = "DESCRIBE $c_table";

        $result = db_query( $query );

        $count = db_num_rows( $result );

        for ( $i=0 ; $i < $count ; $i++ ) {
            $row = db_fetch_array( $result );

            if ( $row['Field'] == $c_field ) {
                return ( $row['Key'] == $c_key );
            }
        }

        return false;
    }


    # --------------------
    # close the connection.
    # Not really necessary most of the time since a connection is
    # automatically closed when a page finishes loading.
    function db_close() {
        $t_result = pg_close();
    }
    function db_error() {
        return pg_last_error();
    }
    # --------------------
    # prepare a string before DB insertion
    function db_prepare_string( $p_string ) {
    $p_string = string_demicrosoftize( $p_string);
    $p_string = trim( $p_string);
        return pg_escape_string( $p_string );
    }

    # --------------------
    # prepare an integer before DB insertion
    function db_prepare_int( $p_int ) {
        return (integer)$p_int;
    }

    # --------------------
    # prepare a boolean before DB insertion
    function db_prepare_bool( $p_bool ) {
        return (int)(bool)$p_bool;
    }
  
    $res=db_connect( $g_pg_hostname, $g_pg_db_user, $g_pg_db_pwd, $g_pg_port, $g_pg_db_name );
?>
