<?php
//Définition projection en vigueur : IGNF car plus précise
$g_tab_srid=array(
    'IGNF:LAMB93' => 'France métropolitaine (IGNF:LAMB93)',
    'IGNF:UTM20W84GUAD'=>'Guadeloupe (IGNF:UTM20W84GUAD)',
    'IGNF:UTM20W84MART'=>'Martinique (IGNF:UTM20W84MART)',
    'IGNF:RGR92UTM40S'=>'Réunion (IGNF:RGR92UTM40S)',
    'IGNF:UTM22RGFG95'=>'Guyane (IGNF:UTM22RGFG95)',
    'IGNF:RGM04UTM38S'=>'Mayotte (IGNF:RGM04UTM38S)', 
    'IGNF:RGSPM06U21'=>'Saint-Pierre-et-Miquelon (IGNF:RGSPM06U21)'
   );
//Datum/Projection des fichiers MapInfo extrait depuis ogrinfo
$g_proj_mapinfo = array(
    'NTF_Paris_Meridian/Lambert_Conformal_Conic_2SP' => 'Lambert 2E',
    'WGS_1984/Lambert_Conformal_Conic_2SP' => 'Lambert 93',
    'GRS_80/Lambert_Conformal_Conic_2SP'=>'Lambert 93'
    );
//Correspondance Zone UTM / Central_meridian de l'ogrinfo : permet de retrouver proj DOMs
$g_proj_dom = array(
    '-63' => 'UTM20',
    '-51' => 'UTM22',
    '57' => 'UTM40'
    );
//Correspondance Nord ou Sud / false_northing de l'ogrinfo : permet de retrouver proj DOMs
$g_proj_ns = array(
    '10000000'=>'S',
    '0'=>'N');
    
// Retour d'erreur de l'ogrinfo si pas de projection ou autres
$g_retour_get_proj= array(
            0 => 'Lien symbolique : non modifiable',
            1 => 'Erreur, le fichier n\'existe pas',
            2 => 'Erreur, impossible de récupérer la projection',
            3 => 'Erreur, pas de projection (Nonearth)',
            4 => '',
            5 => 'Aucune projection'
            );

require_once('config.php');
global $param;

$g_bin_ogrinfo =  '/geobase/local/bin/ogrinfo';
$g_bin_ogr2ogr =  '/geobase/local/bin/ogr2ogr';

function get_proj($nom, $chemin){
    global $g_bin_ogrinfo, $param, $g_proj_dom, $g_proj_ns;
//echo $param['chemin_geobase'].$chemin.'/'.$nom.'.TAB';
    if (!is_link($param['chemin_geobase'].$chemin.'/'.$nom.'.TAB')){
        if (file_exists($param['chemin_geobase'].$chemin.'/'.$nom.'.TAB')){

            $cmd = $g_bin_ogrinfo.' '.$param['chemin_geobase'].$chemin.'/'.$nom.'.TAB '.$nom.' -fid 0';
            exec($cmd, $output, $return_var);
            if ($return_var==0){
                $proj = 'Aucune';
                $datum = 'Aucun';
                $projDOM = array();
                for ($i=1;$i<count($output);$i++){
                    if (isset($output[$i])){
                        if (preg_match('/.*PROJECTION\["(.*)"\].*/', $output[$i], $matches)){
                            $proj = $matches[1];
                        }
                        if (preg_match('/.*DATUM\["(.*)"\.*/', $output[$i], $matches)){
                            $datum = $matches[1];
                        }
                        if (preg_match('/.*Nonearth.*/', $output[$i], $matches)){
                            $proj = 'Nonearth';
                        }
                        if (preg_match('/.*PARAMETER\["central_meridian",(.*)],/', $output[$i], $matches)){
                            $projDOM[0] = $matches[1];
                        }
                        if (preg_match('/.*PARAMETER\["false_northing",(.*)],/', $output[$i], $matches)){
                            $projDOM[1] = $matches[1];
                        }
                    }
                }
                if ($proj == 'Aucune' and $datum == 'Aucun'){
                    return 5;
                }elseif($datum<>'Aucun' and $proj == 'Aucune'){
                    return $datum;
                }elseif($proj == 'Nonearth'){
                    return 3;
                }else{
                    if ($datum.'/'.$proj == 'WGS_1984/Transverse_Mercator'){
                        return  $datum.'/'.$proj.'/'.$g_proj_dom[$projDOM[0]].$g_proj_ns[$projDOM[1]];
                    }else{
                        return $datum.'/'.$proj;
                    }
                }
            }else{
                return 2;
            }
         }else{
            return 1;
        }
    }else{
        return 0;
    }
}

function affiche_list_proj(){
    global $g_tab_srid;
    ?>
    <select name="lst_proj">
        <?php
        foreach ($g_tab_srid as $key=>$value){
            if(substr($key,0,3)=="opt"){
                ?>
                <optgroup label="<?php echo $value ?>">
                <?php
            }elseif(substr($key,0,4)=="copt"){
                ?>
                </optgroup>
                <?php
            }else{
                ?>
                <option value="<?php echo $key ?>"><?php echo $value ?></option>
                <?php
            }
        }
        ?>
    </select>
    <?php
}

function reproj($cheminCouche, $nomCouche , $proj){
    global $g_bin_ogr2ogr, $param;
    
    $formats = array('.TAB', '.DAT', '.ID', '.IND', '.MAP');
    
    $nom = explode('.', $nomCouche);
    
    $proj_orig = get_proj($nom[0], str_replace($param['chemin_geobase'], '', $cheminCouche));
    if ($proj_orig == 'WGS_1984/Lambert_Conformal_Conic_2SP'){
        $ret = ' => déjà en LAMB93';
    }elseif(preg_match('/.*_OLD\..*/', $nomCouche, $matches)){
        $ret = ' => fichier de sauvegarde OLD non transformable';
    }else{
        //ogr2ogr -f "MapInfo File" -s_srs proj_depart -t_srs proj_destination fichier_reprojete fichier_orig dataset_orig
        //ogr2ogr -f "MapInfo File" -t_srs proj_destination fichier_reprojete fichier_orig dataset_orig
        
        //Sauvegarde des fichiers dans un endroit sûr
	$resMv = 0;
        foreach($formats as $formatFile){
            $cmd = 'mv '.$cheminCouche.'/'.$nom[0].$formatFile.' '.$cheminCouche.'/'.$nom[0].'_OLD'.$formatFile;
            //echo $cmd;
            exec($cmd, $output, $return_var);
	    $resMv = $resMv + $return_var;
        }
	if ($param['debug']){
		if ($resMv==0){
			echo 'Déplacement fichier en .OLD réussi<br />';
		}else{
			echo 'Déplacement fichier en .OLD échoué<br />';
		}
	}
        
        //reprojection
        if ($proj_orig == 'NTF_Paris_Meridian/Lambert_Conformal_Conic_2SP'){
	    //Cas Lambert 2E
            $cmd = $g_bin_ogr2ogr.' -f "MapInfo File" -s_srs "+init=IGNF:LAMBE +wktext" -t_srs "+init='.$proj.'" '.$cheminCouche.'/'.$nom[0].'_inter.TAB '.$cheminCouche.'/'.$nom[0].'_OLD.TAB '.$nom[0].'_OLD';
            $cmd2 = $g_bin_ogr2ogr.' -f "MapInfo File" -a_srs '.$param['chemin_prj'].' '.$cheminCouche.'/'.$nom[0].'.TAB '.$cheminCouche.'/'.$nom[0].'_inter.tab '.$nom[0].'_inter';
	    if ($param['debug']){
                echo 'Reprojection depuis L2E:<br />'.$cmd.'<br />'.$cmd2;
            }
        }else{
	    //Cas autre
            $cmd = $g_bin_ogr2ogr.' -f "MapInfo File" -t_srs "+init='.$proj.'" '.$cheminCouche.'/'.$nom[0].'.TAB '.$cheminCouche.'/'.$nom[0].'_OLD.TAB '.$nom[0].'_OLD';
            $cmd2 = ''; 
 	    if ($param['debug']){
                echo 'Reprojection autre:<br />'.$cmd;
            }
        }
   
        exec($cmd, $output, $return_var);
        if ($return_var==0){
	    if ($param['debug']){
		echo 'cmd1 réussie<br />';
	    }
            $ret = ' => réussi !';
            
            if ($cmd2 != ''){
                //echo $cmd2;
                exec($cmd2, $output, $return_var);

                if ($return_var!=0){
                    $ret = ' => reprojection échouée !';
           		if ($param['debug']){
	                   echo 'cmd2 échouée<br />';
			   print_r($output);
           		 } 
                    //On rétablit les fichiers OLD
                    foreach($formats as $formatFile){
                        $cmd = 'mv '.$cheminCouche.'/'.$nom[0].'_OLD'.$formatFile.' '.$cheminCouche.'/'.$nom[0].$formatFile;
                        //echo $cmd;
                        exec($cmd, $output, $return_var);
                    }
		    
                }
                
                #Suppression des fichiers intermédiaires
                $nom = explode('.', $nomCouche);
                foreach($formats as $formatFile){
                    $cmd = 'rm '.$cheminCouche.'/'.$nom[0].'_inter'.strtolower($formatFile);
                    exec($cmd, $output, $return_var);
                }
            }
            
            //Renommage du fichier pour passer les extensions en majuscule
            $nom = explode('.', $nomCouche);
            foreach($formats as $formatFile){
                $cmd = 'mv '.$cheminCouche.'/'.$nom[0].strtolower($formatFile).' '.$cheminCouche.'/'.$nom[0].$formatFile;
                exec($cmd, $output, $return_var);
            }
            
        }else{
            $ret = ' => reprojection échouée !';
            if ($param['debug']){
                echo 'cmd1 échouée<br />';
            }
            //On rétablit les fichiers OLD
            foreach($formats as $formatFile){
                $cmd = 'mv '.$cheminCouche.'/'.$nom[0].'_OLD'.$formatFile.' '.$cheminCouche.'/'.$nom[0].$formatFile;
                //echo $cmd;
                exec($cmd, $output, $return_var);
            }
        }
        
    }
    return $ret;
}

?>
