<?php
    # GeoREP - Gestionnaire des couches de la GoBASE
    # Copyright (C) 2004 - Michel Wurtz - MAAPAR/SDSI/CERI/DIG
    # bas sur Mantis (http://www.mantisbt.org)
    # Copyright (C) 2000 - 2002 Kenzaburo Ito - kenito@300baud.org
    # Copyright (C) 2002 - 2004 Mantis Team - mantisbt-dev@lists.sourceforge.net
    # This program is distributed under the terms and conditions of the GPL
    # See the README and LICENSE files for details

    # --------------------------------------------------------
    # $Id: string_api.php,v 1.43 2004/02/22 04:41:46 vboctor Exp $
    # --------------------------------------------------------

 

    ###########################################################################
    # String Processing API
    ###########################################################################

    ### --------------------
    # Preserve spaces at beginning of lines.
    # Lines must be separated by \n rather than <br />
    function string_preserve_spaces_at_bol( $p_string ) {
        $lines = explode("\n", $p_string);
        for ( $i = 0; $i < count( $lines ); $i++ ) {
            $count = 0;
            $prefix = '';
            while ( substr($lines[$i], $count, 1) == ' ' ) {
              $count++;
            }
            for ($j = 0; $j < $count; $j++) {
              $prefix .= '&nbsp;';
            }
            $lines[$i] = $prefix . substr( $lines[$i], $count );

        }
        $result = implode( "\n", $lines );
        return $result;
    }
    # --------------------
    # Prepare a string for display to HTML
    function string_display( $p_string ) {
	$p_string = string_demicrosoftize( $p_string);
        $p_string = string_strip_hrefs( $p_string );
        $p_string = htmlspecialchars( $p_string );
        $p_string = string_restore_valid_html_tags( $p_string );
        $p_string = string_preserve_spaces_at_bol( $p_string );
        $p_string = nl2br( $p_string );

        return $p_string;
    }
    
    # --------------------
    # Prepare a string for display to HTML and add href anchors for URLs,
    # emails and layer references
    function string_display_links( $p_string ) {
        $p_string = string_display( $p_string );

        $p_string = string_insert_hrefs( $p_string );
        $p_string = string_process_layer_link( $p_string );
        
        return $p_string;
    }

    # --------------------
    # Prepare a string for plain text display in email
    function string_email( $p_string ) {
	$p_string = string_demicrosoftize( $p_string);
        $p_string = string_strip_hrefs( $p_string );
        return $p_string;
    }

    # --------------------
    # Remplace les caractres dbiles par des choses lisibles
    function string_demicrosoftize( $str ) {

	$unicode_array = array(
	    "&#8211;" => "-",
	    "&#8212;" => "-",
	    "&#8216;" => "'",
	    "&#8217;" => "'",
	    "&#8230;" => "...",
	    "&#8220;" => "\"",
	    "&#8221;" => "\"",
	); 
	$my_string = strtr($str, $unicode_array);
	return strtr($my_string,
	    "\x82\x83\x84\x85\x86\x87\x89\x8a\x8b\x8c\x8e\x91\x92\x93\x94\x95" .
    	"\x96\x97\x98\x99\x9a\x9b\x9c\x9e\x9f",
   	 "'f\".**^\xa6<\xbc\xb4''\"\"---~ \xa8>\xbd\xb8\xbe");
    }
    
    # --------------------
    # Prepare a string for plain text display in email and add URLs for layer
    #  links  
    function string_email_links( $p_string ) {
        $p_string = string_email( $p_string );
        
        $p_string = string_process_layer_link( $p_string, false );
        
        return $p_string;
    }

    # --------------------
    # Process a string for display in a textarea box
    function string_textarea( $p_string ) {
	$p_string = string_demicrosoftize( $p_string);
        $p_string = htmlspecialchars( $p_string );
        return $p_string;
    }

    # --------------------
    # Process a string for display in a text box
    function string_attribute( $p_string ) {
	$p_string = string_demicrosoftize( $p_string);
        $p_string = htmlspecialchars( $p_string );
        return $p_string;
    }

    # --------------------
    # Process a string for inclusion in a URL as a GET parameter
    function string_url( $p_string ) {
        $p_string = rawurlencode( $p_string );
        return $p_string;
    }

    # --------------------
    # Prepare une adresse URL venant de REPORTS 2003
    function prepare_url_reports( $p_url ) {
	$t_balise = array (
	    "#" => "",
	    "\$" => " (",
	    "|" => ")", );
	$c_url = strtr( $p_url, $t_balise );
        return $c_url;
    }
    
    # --------------------
    # Process $p_string, looking for layer ID references and creating layer view
    #  links for them.
    #
    # Returns the processed string.
    #
    # If $p_include_anchor is true, include the href tag, otherwise just insert
    #  the URL
    #
    # The layer tag ('#' by default) must be at the beginning of the string or
    #  preceeded by a character that is not a letter, a number or an underscore
    function string_process_layer_link( $p_string, $p_include_anchor=true ) {
        $t_tag = config_get( 'layer_link_tag' );
        $t_path = config_get( 'path' );

        preg_match_all( '/(^|.+?)(?:(?<=^|\W)' . preg_quote($t_tag) . '(\d+)|$)/s',
                                $p_string, $t_matches, PREG_SET_ORDER );
        
        $t_result = '';

        if ( $p_include_anchor ) {
            foreach ( $t_matches as $t_match ) {
                $t_result .= $t_match[1];

                if ( isset( $t_match[2] ) ) {
                    $t_layer_id = $t_match[2];
                    if ( layer_exists( $t_layer_id ) ) {
                        $t_result .= string_get_layer_view_link( $t_layer_id );
                    } else {
                        $t_result .= $t_layer_id;
                    }
                }
            }
        } else {
            foreach ( $t_matches as $t_match ) {
                $t_result .= $t_match[1];
                
                if ( isset( $t_match[2] ) ) {
                    $t_layer_id = $t_match[2];
                    # We might as well create the link here even if the layer
                    #  doesn't exist.  In the case above we don't want to do
                    #  the summary lookup on a non-existant layer.  But here, we
                    #  can create the link and by the time it is clicked on, the
                    #  layer may exist.

                    $t_result .= string_get_layer_view_url_with_fqdn( $t_layer_id );
                }
            }
        }

        return $t_result;
    }

    #===================================
    # Tag Processing
    #===================================

    # --------------------
    # Detect URLs and email addresses in the string and replace them with href anchors
    function string_insert_hrefs( $p_string ) {
        if ( ! config_get( 'html_make_links' ) ) {
            return $p_string;
        }

        # This is based on the description in RFC 2396 which specifies how
        #  to match URLs generically without knowing their type
        $p_string = preg_replace( '/(([[:alpha:]][-+.[:alnum:]]*):\/\/(%[[:digit:]A-Fa-f]{2}|[-_.!~*\';\/?:@&=+$,[:alnum:]])+)/s',
                                '<a href="\1">\1</a>',
                                $p_string);

        # Set up a simple subset of RFC 822 email address parsing
        #  We don't allow domain literals or quoted strings
        #  We also don't allow the & character in domains even though the RFC
        #  appears to do so.  This was to prevent &gt; etc from being included.
        #  Note: we could use email_get_rfc822_regex() but it doesn't work well
        #  when applied to data that has already had entities inserted.
        $t_atom = '(?:[^()<>@,;:\\\".\[\]\000-\037\177 &]+)';

        # In order to avoid selecting URLs containing @ characters as email
        #  addresses we limit our selection to addresses that are preceded by:
        #  * the beginning of the string
        #  * a &lt; entity (allowing '<foo@bar.baz>')
        #  * whitespace
        #  * a : (allowing 'send email to:foo@bar.baz')
        #  * a \n, \r, or > (because newlines have been replaced with <br />
        #    and > isn't valid in URLs anyway
        #
        # At the end of the string we allow the opposite:
        #  * the end of the string
        #  * a &gt; entity
        #  * whitespace
        #  * a , character (allowing 'email foo@bar.baz, or ...')
        #  * a \n, \r, or <
        $p_string = preg_replace( '/(?<=^|&lt;|[\s\:\>\n\r])('.$t_atom.'(?:\.'.$t_atom.')*\@'.$t_atom.'(?:\.'.$t_atom.')*)(?=$|&gt;|[\s\,\<\n\r])/s',
                                '<a href="mailto:\1" target="_new">\1</a>',
                                $p_string);
        return $p_string;
    }

    # --------------------
    # Detect href anchors in the string and replace them with URLs and email addresses
    function string_strip_hrefs( $p_string ) {
        # First grab mailto: hrefs.  We don't care whether the URL is actually
        # correct - just that it's inside an href attribute.
        $p_string = preg_replace( '/<a\s[^\>]*href="mailto:([^\"]+)"[^\>]*>[^\<]*<\/a>/si',
                                '\1',
                                $p_string);

        # Then grab any other href
        $p_string = preg_replace( '/<a\s[^\>]*href="([^\"]+)"[^\>]*>[^\<]*<\/a>/si',
                                '\1',
                                $p_string);
        return $p_string;
    }

    # --------------------
    # This function looks for text with htmlentities
    # like &lt;b&gt; and converts is into corresponding
    # html <b> based on the configuration presets
    function string_restore_valid_html_tags( $p_string ) {
        $t_html_valid_tags = config_get( 'html_valid_tags' );

        if ( OFF === $t_html_valid_tags ||
             is_blank( $t_html_valid_tags ) ) {
            return $p_string;
        }

        $tags = explode( ',', $t_html_valid_tags );

        foreach ( $tags as $tag ) {
            if ( !is_blank( $tag ) ) {
                $tag = trim( $tag );
                $p_string = eregi_replace( "&lt;($tag)[[:space:]]*&gt;", "<\\1>", $p_string );
                $p_string = eregi_replace( "&lt;\/($tag)[[:space:]]*&gt;", "</\\1>", $p_string );
                $p_string = eregi_replace( "&lt;($tag)[[:space:]]*\/&gt;", "<\\1 />", $p_string );
            }
        }

        return $p_string;
    }


    #===================================
    # Advanced/Simple page selection
    #===================================

    # --------------------
    # return an href anchor that links to a layer VIEW page for the given layer
    function string_get_layer_view_link( $p_layer_id ) {
        if ( layer_exists( $p_layer_id ) ) {
            $t_summary = string_attribute( layer_get_field( $p_layer_id, 'id' ) );
            $t_status = string_attribute( get_enum_element( 'status', demande_get_field( $p_layer_id, 'status' ) ) );
            return '<a href="' . string_get_layer_view_url( $p_layer_id ) . '" title="[' . $t_status . '] ' . $t_summary . '">' . layer_format_id( $p_layer_id ) . '</a>';
        } else {
            return layer_format_id( $p_layer_id );
        }
    }

    # --------------------
    # return the name and GET parameters of a layer VIEW page for the given layer
    function string_get_layer_view_url( $p_layer_id ) {
        return 'layer_view_page.php?layer_id=' . layer_format_id( $p_layer_id );
    }

    # --------------------
    # return the name and GET parameters of a layer VIEW page for the given layer
    # The returned url includes the fully qualified domain, hence it is suitable to be included
    # in emails.
    function string_get_layer_view_url_with_fqdn( $p_layer_id ) {
        return config_get( 'path' ) . string_get_layer_view_url( $p_layer_id );
    }

    # --------------------
    # return the name and GET parameters of a layer UPDATE page for the given layer
    #  account for the user preference and site override
    function string_get_layer_update_url( $p_layer_id ) {
        return string_get_layer_update_page() . '?layer_id=' . layer_format_id( $p_layer_id );
    }

    # --------------------
    # return the name of a layer UPDATE page for the user
    #  account for the user preference and site override
    function string_get_layer_update_page() {
        return 'layer_update_page.php';
    }

    # --------------------
    # transforme une chaine en remplaant les tags html
    # utilis pour l'export XML (en pensant  REPORTS)
    function utf8_xml($s) {
	$orig = array( "<ul>", "</ul>", "<ol>", "</ol>", "<li>", "</li>", "<br>", "<br />", "<p>", "</p>", "<b>", "</b>", "<i>", "</i>", "<u>", "</u>" );
	$final = array( "\n", "\n", "\n", "\n", "\n* ", "", "\n", "\n", "\n", "\n", "");
	$s = str_replace( $orig, $final, $s);
	$s = html_entity_decode ( $s, ENT_QUOTES );
	$s = utf8_encode( $s);
	return string_email_links( $s);
}


    # --------------------
    # met une chaine sous forme de bloc de $p_len caractres,
    # la marge gauche ayant $p_pad blanc...
    # $p_first controle le dcalage de la premire ligne
    function new_wrap( $p_text, $p_len, $p_pad, $p_first=1 ) {
        $g_text_pad = str_pad('', $p_pad, ' ');
	$c_text = wordwrap( $p_text, $p_len );
	if ( $p_first )
	    return $g_text_pad.str_replace( "\n", "\n" . $g_text_pad, $c_text );
	else
	    return str_replace( "\n", "\n" . $g_text_pad, $c_text );
    }
?>
