
CREATE ROLE geomap WITH LOGIN
  NOSUPERUSER INHERIT CREATEDB CREATEROLE 
	ENCRYPTED PASSWORD 'dig-geobase' ;

CREATE SCHEMA gb_echange
  AUTHORIZATION geomap;

CREATE TABLE gb_echange.utilisateur
(
  id_utilisateur integer NOT NULL DEFAULT nextval(('gb_echange.utilisateur_idutilisateur_seq'::text)::regclass),
  nom character varying(45) NOT NULL DEFAULT ''::character varying,
  prenom character varying(45) NOT NULL DEFAULT ''::character varying,
  "admin" integer NOT NULL DEFAULT 0,
  service character varying(30),
  adresse character varying(30),
  code_postal character varying(30),
  ville character varying(30),
  "login" character varying(150),
  CONSTRAINT pk_idutilisateur PRIMARY KEY (id_utilisateur)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE gb_echange.utilisateur OWNER TO geomap;

CREATE TABLE gb_echange.gb_export
(
  nom_couche character varying(32) NOT NULL, -- Nom de la couche selon la description locale dans GéoREPERTOIRE
  serveur character varying(50) NOT NULL, -- Nom du serveur autorisé à importer la donnée
  id_export integer NOT NULL DEFAULT nextval(('gb_echange.gb_export_id_export_seq'::text)::regclass), -- Identifiant
  nom_serveur character varying(150),
  CONSTRAINT pk_id_export PRIMARY KEY (id_export)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE gb_echange.gb_export OWNER TO geomap;
COMMENT ON TABLE gb_echange.gb_export IS 'Table des couches exportables';
COMMENT ON COLUMN gb_echange.gb_export.nom_couche IS 'Nom de la couche selon la description locale dans GéoREPERTOIRE';
COMMENT ON COLUMN gb_echange.gb_export.serveur IS 'Nom du serveur autorisé à importer la donnée';
COMMENT ON COLUMN gb_echange.gb_export.id_export IS 'Identifiant';

CREATE TABLE gb_echange.gb_import
(
  nom_couche character varying(32) NOT NULL, -- Nom de la couche selon la description locale dans GéoREPERTOIRE
  serveur character varying(50) NOT NULL, -- Nom du serveur où cherche la donnée
  date_import timestamp without time zone, -- Date de dernier import avec succès
  md5_tab character(32), -- md5sum du fichier .TAB
  md5_map character(32), -- md5sum du fcichier .MAP
  md5_dat character(32), -- md5sum du fichier .DAT
  archiver boolean NOT NULL DEFAULT false, -- Indique si la donnée locale doit être archivée avant insertion de la donnée importée
  nom_serveur character varying(150),
  log_import character varying(100),
  CONSTRAINT pk_nom_couche PRIMARY KEY (nom_couche)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE gb_echange.gb_import OWNER TO geomap;
COMMENT ON TABLE gb_echange.gb_import IS 'Liste des couches à importer';
COMMENT ON COLUMN gb_echange.gb_import.nom_couche IS 'Nom de la couche selon la description locale dans GéoREPERTOIRE';
COMMENT ON COLUMN gb_echange.gb_import.serveur IS 'Nom du serveur où cherche la donnée';
COMMENT ON COLUMN gb_echange.gb_import.date_import IS 'Date de dernier import avec succès';
COMMENT ON COLUMN gb_echange.gb_import.md5_tab IS 'md5sum du fichier .TAB';
COMMENT ON COLUMN gb_echange.gb_import.md5_map IS 'md5sum du fcichier .MAP';
COMMENT ON COLUMN gb_echange.gb_import.md5_dat IS 'md5sum du fichier .DAT';
COMMENT ON COLUMN gb_echange.gb_import.archiver IS 'Indique si la donnée locale doit être archivée avant insertion de la donnée importée';


